function [theta_hat, Gamma, rho, kappa] = OL_qSPICE(y, A, q, iter,regu)
%   Code for regularized online SPICE, see the article "Online Sparse Reconstruction for Scanning Radar Using Beam-updating q-SPICE"
%   Original code is developed by Adham Sakhnini, see the article "Time-updating the Generalized Sparse Covariance-based Estimator"
%   input: y_n------- input data of length n
%          h_n------- h_n = A(n, :); h_n = h_n'; where A is the steering matrix
%          q--------- norm
%          iter------ iteration number
%          Gamma----- temporary variable;
%          rho------- temporary variable;
%          kappa----- temporary variable;
%          theta_hat- resulting estimates;
%          n--------- order number of input data;
[M,~]=size(y);
[~,K]=size(A);
theta_hat = ones(K,1);
Gamma = zeros(K,K);
rho   = zeros(K,1);
kappa = zeros(1,1);
for n=1:M
    y_n = y(n);
    h_n = A(n, :);
    h_n = h_n';
    % Update online parameters
    Gamma = Gamma + h_n * h_n';
    rho = rho + h_n * y_n;
    kappa = kappa + y_n' * y_n;
    
    % Set instance parameters
    nu = kappa + theta_hat' * Gamma * theta_hat - 2 * real(theta_hat' * rho);
    eta = rho - Gamma * theta_hat;
    for k = 1:iter
        for i = 1:K
            G = Gamma(i, i);
            th = theta_hat(i);
            et = eta(i);
            %alpha = nu + Gamma(i, i) * theta_hat(i)' * theta_hat(i) + 2 * real(theta_hat(i)' * eta(i));
            alpha = nu + G * (th') * th + 2 * real(th' * et);
            %beta = Gamma(i, i);
            beta = G+regu;
            comp = et + G*th;
            %gamma = abs(eta(i) + Gamma(i, i) * theta_hat(i));
            g = abs(comp);
            % Estimate angle
            %phi = angle(eta(i) + Gamma(i, i) * theta_hat(i));
            ph = angle(comp);
            % Estimate range
            
            if sqrt(n^(1/q) - 1) * g > sqrt(alpha*beta - g^2)
                %r = gamma / beta - (1 / beta) * sqrt((alpha * beta - gamma^2) / (n^(1/q) - 1));
                r = g/(beta) - (1/(beta)) * sqrt((alpha*beta - g^2)/(n^(1/q) - 1));
            else
                r = 0;
            end
            % Parameter estimate
            new_theta = r * exp(1i * ph);
            % Update instance parameters
            %nu = nu + Gamma(i, i) * abs(theta_hat(i) - new_theta)^2 + 2 * real((theta_hat(i) - new_theta)' * eta(i));
            nu = nu + G * abs(th - new_theta)^2 + 2 * real((th - new_theta)' * et);
            eta = eta + Gamma(:, i) * (th - new_theta);
            theta_hat(i) = new_theta;
        end
    end
end
end

